@@ -21,9 +21,11 @@ from typing import (
     cast,
 )
 from functools import lru_cache
+from typing_extensions import Literal
 
 import anyio
 import httpx
+import distro
 import pydantic
 from httpx import URL
 from pydantic import PrivateAttr
@@ -44,6 +46,7 @@ from ._types import (
     RequestOptions,
     ModelBuilderProtocol,
 )
+from ._utils import is_dict
 from ._models import BaseModel, GenericModel, FinalRequestOptions
 from ._base_exceptions import (
     APIStatusError,
@@ -275,6 +278,8 @@ class BaseClient:
                 options.idempotency_key = self._idempotency_key()
             headers[self._idempotency_header] = options.idempotency_key
 
+        kwargs: dict[str, Any] = {}
+
         # If the given Content-Type header is multipart/form-data then it
         # has to be removed so that httpx can generate the header with
         # additional information for us as it has to be in this form
@@ -283,6 +288,15 @@ class BaseClient:
         if headers.get("Content-Type") == "multipart/form-data":
             headers.pop("Content-Type")
 
+            # As we are now sending multipart/form-data instead of application/json
+            # we need to tell httpx to use it, https://www.python-httpx.org/advanced/#multipart-file-encoding
+            if options.json_data:
+                if not is_dict(options.json_data):
+                    raise TypeError(
+                        f"Expected query input to be a dictionary for multipart requests but got {type(options.json_data)} instead."
+                    )
+                kwargs["data"] = self._serialize_multipartform(options.json_data)
+
         # TODO: report this error to httpx
         return self._client.build_request(  # pyright: ignore[reportUnknownMemberType]
             headers=headers,
@@ -296,8 +310,23 @@ class BaseClient:
             params=self.qs.stringify(cast(Mapping[str, Any], options.params)) if options.params else None,
             json=options.json_data,
             files=options.files,
+            **kwargs,
         )
 
+    def _serialize_multipartform(self, data: Mapping[object, object]) -> dict[str, object]:
+        items = self.qs.stringify_items(
+            # TODO: type ignore is required as stringify_items is well typed but we can't be
+            # well typed without heavy validation.
+            data,  # type: ignore
+            array_format="brackets",
+        )
+        serialized: dict[str, object] = {}
+        for key, value in items:
+            if key in serialized:
+                raise ValueError(f"Duplicate key encountered: {key}; This behaviour is not supported")
+            serialized[key] = value
+        return serialized
+
     def process_response(
         self,
         *,
@@ -355,12 +384,21 @@ class BaseClient:
     def qs(self) -> Querystring:
         return Querystring()
 
+    @property
+    def custom_auth(self) -> httpx.Auth | None:
+        return None
+
+    @property
+    def auth_headers(self) -> Dict[str, str]:
+        return {}
+
     @property
     def default_headers(self) -> Dict[str, str]:
         return {
             "Content-Type": "application/json",
             "User-Agent": self.user_agent,
-            "X-Stainless-Client-User-Agent": self.platform_properties(),
+            **self.platform_headers(),
+            **self.auth_headers,
         }
 
     @property
@@ -372,17 +410,15 @@ class BaseClient:
         return self._client.base_url
 
     @lru_cache(maxsize=None)
-    def platform_properties(self) -> str:
-        arch, _ = platform.architecture()
-        properties = {
-            "lang": "python",
-            "packageVersion": self._version,
-            "os": platform.system(),
-            "arch": arch,
-            "runtime": platform.python_implementation(),
-            "runtimeVersion": platform.python_version(),
+    def platform_headers(self) -> Dict[str, str]:
+        return {
+            "X-Stainless-Lang": "python",
+            "X-Stainless-Package-Version": self._version,
+            "X-Stainless-OS": str(get_platform()),
+            "X-Stainless-Arch": str(get_architecture()),
+            "X-Stainless-Runtime": platform.python_implementation(),
+            "X-Stainless-Runtime-Version": platform.python_version(),
         }
-        return json.dumps(properties)
 
     def calculate_retry_timeout(
         self,
@@ -479,7 +515,7 @@ class SyncAPIClient(BaseClient):
         request = self.build_request(options)
 
         try:
-            response = self._client.send(request)
+            response = self._client.send(request, auth=self.custom_auth)
             response.raise_for_status()
         except httpx.HTTPStatusError as err:  # thrown on 4xx and 5xx status code
             if retries > 0 and self.should_retry(err.response):
@@ -539,10 +575,9 @@ class SyncAPIClient(BaseClient):
         path: str,
         *,
         cast_to: Type[ResponseT],
-        query: Query = {},
         options: RequestOptions = {},
     ) -> ResponseT:
-        opts = FinalRequestOptions(method="get", url=path, params=query, **options)
+        opts = FinalRequestOptions.construct(method="get", url=path, **options)
         return self.request(cast_to, opts)
 
     def post(
@@ -554,7 +589,7 @@ class SyncAPIClient(BaseClient):
         options: RequestOptions = {},
         files: RequestFiles | None = None,
     ) -> ResponseT:
-        opts = FinalRequestOptions(method="post", url=path, json_data=body, files=files, **options)
+        opts = FinalRequestOptions.construct(method="post", url=path, json_data=body, files=files, **options)
         return self.request(cast_to, opts)
 
     def patch(
@@ -565,7 +600,7 @@ class SyncAPIClient(BaseClient):
         body: Query | None = None,
         options: RequestOptions = {},
     ) -> ResponseT:
-        opts = FinalRequestOptions(method="patch", url=path, json_data=body, **options)
+        opts = FinalRequestOptions.construct(method="patch", url=path, json_data=body, **options)
         return self.request(cast_to, opts)
 
     def put(
@@ -576,7 +611,7 @@ class SyncAPIClient(BaseClient):
         body: Query | None = None,
         options: RequestOptions = {},
     ) -> ResponseT:
-        opts = FinalRequestOptions(method="put", url=path, json_data=body, **options)
+        opts = FinalRequestOptions.construct(method="put", url=path, json_data=body, **options)
         return self.request(cast_to, opts)
 
     def delete(
@@ -587,7 +622,7 @@ class SyncAPIClient(BaseClient):
         body: Query | None = None,
         options: RequestOptions = {},
     ) -> ResponseT:
-        opts = FinalRequestOptions(method="delete", url=path, json_data=body, **options)
+        opts = FinalRequestOptions.construct(method="delete", url=path, json_data=body, **options)
         return self.request(cast_to, opts)
 
     def get_api_list(
@@ -596,10 +631,9 @@ class SyncAPIClient(BaseClient):
         *,
         model: Type[ModelT],
         page: Type[SyncPageT],
-        query: Query = {},
         options: RequestOptions = {},
     ) -> SyncPageT:
-        opts = FinalRequestOptions(method="get", url=path, params=query, **options)
+        opts = FinalRequestOptions.construct(method="get", url=path, **options)
         return self.request_api_list(model, page, opts)
 
 
@@ -637,7 +671,7 @@ class AsyncAPIClient(BaseClient):
         request = self.build_request(options)
 
         try:
-            response = await self._client.send(request)
+            response = await self._client.send(request, auth=self.custom_auth)
             response.raise_for_status()
         except httpx.HTTPStatusError as err:  # thrown on 4xx and 5xx status code
             if retries > 0 and self.should_retry(err.response):
@@ -702,7 +736,7 @@ class AsyncAPIClient(BaseClient):
         query: Query = {},
         options: RequestOptions = {},
     ) -> ResponseT:
-        opts = FinalRequestOptions(method="get", url=path, params=query, **options)
+        opts = FinalRequestOptions.construct(method="get", url=path, **options)
         return await self.request(cast_to, opts)
 
     async def post(
@@ -714,7 +748,7 @@ class AsyncAPIClient(BaseClient):
         files: RequestFiles | None = None,
         options: RequestOptions = {},
     ) -> ResponseT:
-        opts = FinalRequestOptions(method="post", url=path, json_data=body, files=files, **options)
+        opts = FinalRequestOptions.construct(method="post", url=path, json_data=body, files=files, **options)
         return await self.request(cast_to, opts)
 
     async def patch(
@@ -725,7 +759,7 @@ class AsyncAPIClient(BaseClient):
         body: Query | None = None,
         options: RequestOptions = {},
     ) -> ResponseT:
-        opts = FinalRequestOptions(method="patch", url=path, json_data=body, **options)
+        opts = FinalRequestOptions.construct(method="patch", url=path, json_data=body, **options)
         return await self.request(cast_to, opts)
 
     async def put(
@@ -736,7 +770,7 @@ class AsyncAPIClient(BaseClient):
         body: Query | None = None,
         options: RequestOptions = {},
     ) -> ResponseT:
-        opts = FinalRequestOptions(method="put", url=path, json_data=body, **options)
+        opts = FinalRequestOptions.construct(method="put", url=path, json_data=body, **options)
         return await self.request(cast_to, opts)
 
     async def delete(
@@ -747,7 +781,7 @@ class AsyncAPIClient(BaseClient):
         body: Query | None = None,
         options: RequestOptions = {},
     ) -> ResponseT:
-        opts = FinalRequestOptions(method="delete", url=path, json_data=body, **options)
+        opts = FinalRequestOptions.construct(method="delete", url=path, json_data=body, **options)
         return await self.request(cast_to, opts)
 
     def get_api_list(
@@ -757,10 +791,9 @@ class AsyncAPIClient(BaseClient):
         # TODO: support paginating `str`
         model: Type[ModelT],
         page: Type[AsyncPageT],
-        query: Query = {},
         options: RequestOptions = {},
     ) -> AsyncPaginator[ModelT, AsyncPageT]:
-        opts = FinalRequestOptions(method="get", url=path, params=query, **options)
+        opts = FinalRequestOptions.construct(method="get", url=path, **options)
         return self.request_api_list(model, page, opts)
 
 
@@ -768,6 +801,7 @@ def make_request_options(
     headers: Headers | NotGiven,
     max_retries: int | NotGiven,
     timeout: float | Timeout | None | NotGiven,
+    query: Query | None,
 ) -> RequestOptions:
     """Create a dict of type RequestOptions without keys of NotGiven values."""
     options: RequestOptions = {}
@@ -780,9 +814,107 @@ def make_request_options(
     if not isinstance(timeout, NotGiven):
         options["timeout"] = timeout
 
+    if query is not None:
+        options["params"] = query
+
     return options
 
 
+class OtherPlatform:
+    def __init__(self, name: str) -> None:
+        self.name = name
+
+    def __str__(self) -> str:
+        return f"Other:{self.name}"
+
+
+Platform = Union[
+    OtherPlatform,
+    Literal[
+        "MacOS",
+        "Linux",
+        "Windows",
+        "FreeBSD",
+        "OpenBSD",
+        "iOS",
+        "Android",
+        "Unknown",
+    ],
+]
+
+
+def get_platform() -> Platform:
+    system = platform.system().lower()
+    platform_name = platform.platform().lower()
+    if "iphone" in platform_name or "ipad" in platform_name:
+        # Tested using Python3IDE on an iPhone 11 and Pythonista on an iPad 7
+        # system is Darwin and platform_name is a string like:
+        # - Darwin-21.6.0-iPhone12,1-64bit
+        # - Darwin-21.6.0-iPad7,11-64bit
+        return "iOS"
+
+    if system == "darwin":
+        return "MacOS"
+
+    if system == "windows":
+        return "Windows"
+
+    if "android" in platform_name:
+        # Tested using Pydroid 3
+        # system is Linux and platform_name is a string like 'Linux-5.10.81-android12-9-00001-geba40aecb3b7-ab8534902-aarch64-with-libc'
+        return "Android"
+
+    if system == "linux":
+        # https://distro.readthedocs.io/en/latest/#distro.id
+        distro_id = distro.id()
+        if distro_id == "freebsd":
+            return "FreeBSD"
+
+        if distro_id == "openbsd":
+            return "OpenBSD"
+
+        return "Linux"
+
+    if platform_name:
+        return OtherPlatform(platform_name)
+
+    return "Unknown"
+
+
+class OtherArch:
+    def __init__(self, name: str) -> None:
+        self.name = name
+
+    def __str__(self) -> str:
+        return f"other:{self.name}"
+
+
+Arch = Union[OtherArch, Literal["x32", "x64", "arm", "arm64", "unknown"]]
+
+
+def get_architecture() -> Arch:
+    python_bitness, _ = platform.architecture()
+    machine = platform.machine().lower()
+    if machine in ("arm64", "aarch64"):
+        return "arm64"
+
+    # TODO: untested
+    if machine == "arm":
+        return "arm"
+
+    if machine == "x86_64":
+        return "x64"
+
+    # TODO: untested
+    if python_bitness == "32bit":
+        return "x32"
+
+    if machine:
+        return OtherArch(machine)
+
+    return "unknown"
+
+
 def _merge_mappings(
     obj1: Mapping[_T_co, Union[_T, Omit]],
     obj2: Mapping[_T_co, Union[_T, Omit]],
